;****** Lichtendstufe, Betriebsprogramm *********************************
;*                                                                      *
;*      Ulm, den 5. Oktober 1991 (Sk)                                   *
;*                                                                      *
;*      Release: Mnchen, den 30.03.92 (SK)                             *
;*                                                                      *
;*                                                                      *
;*      Zum Betrieb der CAT-Lichtendstufen                              *
;*      Ermoeglicht 8-kanaligen Phasenanschnittbetrieb                  *
;*      auf einer Netzphase.                                            *
;*                                                                      *
;*      Von der Lichtendstufen-Processorkarte werden die                *
;*      Phasenstecker 1 und 3 benutzt.                                  *
;*                                                                      *
;************************************************************************

;****** Externe Definitionsdateien: *************************************

        include x80517.asm
        include macros.asm
        include konstanten.asm

;****** Externe Routinen und Variablen: *********************************

        extern  eeprom_get              ;Einstellungen aus EEPROM holen
        extern  init_control            ;Einstellungen kontrollieren
        extern  baudcontrol             ;Editmode Baudrate
        extern  testmodecontrol         ;Editmode Testen forcieren
        extern  ledanzinit              ;Strom fr LedAnzeige einstellen
        extern  tastatur                ;Tastaturbearbeitung
        extern  display                 ;Display-Darstellung

        extern  kanal:byte              ;Externe Variablen (aus EEPROM)
        extern  preheat:byte
        extern  kurve:byte
        extern  baudrate:byte
        extern  param:byte

        public  setbaudrate
        public  frame_count
        public  data_count
        public  blink

;****** Hauptprogramm: Reseteinsprung ***********************************

        ljmp    INIT                    ;Sprung zum Main-Programm

;****** Nulldurchgang-Interrupt (Ext. Int.1): Einsprung *****************

        org     13h                     ;INT-1 Einsprung
        ljmp    EXTINT1

;****** SIO-Handler: ****************************************************
;*                                                                      *
;*      R7: Abwaertszaehler fuer Kanalnummer                            *
;*      R6: Anfangskennung zur Unterscheidung des Startbytes            *
;*      R5:                                                             *
;*      R0: Pointer und Zaehler fuer 8 Bytes                            *
;*                                                                      *
;************************************************************************

        org     23h                     ;SIO-Einsprung
        SIOANF
        mov     A,SBUF                  ;Byte von SIO holen
        jnb     RB8,rsframe             ;bei 9.Bit=0 auf Reset testen

;****** Startbyte: ******************************************************

        cjne    R6,#0,normbyt
        inc     R6
        jz      normend
        mov     R0,#lampe+8             ;Kein Null-Startcode,
        inc     R5
        SIOENDE                         ;nichts empfangen

;****** Normales Byte: **************************************************

normbyt cjne    R7,#0,normend
        cjne    R0,#lampe+8,getbyte
        cjne    R5,#0,dataerr
        setb    frame_ok                ;Daten-OK-Meldung an HP
dataerr SIOENDE
normend
        dec     R7
        SIOENDE
getbyte
        mov     @R0,A
        inc     R0
        SIOENDE

;****** Reset (Break): **************************************************

rsframe setb    data_received           ;Daten-Empfangen-Meldung an HP
        jz      resok
        mov     R0,#lampe+8             ;Akku nicht Null -> Datenfehler
        mov     R5,#1
        SIOENDE
resok
        mov     R7,kanal
        mov     R0,#lampe               ;Pointer auf Einlesefeld
        mov     R6,#0                   ;Anfangs-Kennung
        mov     R5,#0
        SIOENDE


;****** Nulldurchgang-Interrupt (Ext. Int.1): Programm ******************
;*                                                                      *
;*                                                                      *
;*                                                                      *
;************************************************************************

EXTINT1
        push    PSW
        push    ACC
        mov     PSW,#INT1BANK

        clr     T2I0                    ;Timer 2 stoppen
        mov     TL2,#0
        mov     TH2,#0                  ;Timer 2 nullsetzen

        mov     CMEN,#0                 ;direkten Zugang zu Portpins
        mov     CCEN,#0
        mov     P4,#0                   ;P4 Portpins loeschen
        mov     A,P1                    ;P1.0-3 Portpins loeschen
        anl     A,#11110000b
        mov     P1,A
        mov     CMEN,#0FFH              ;Portpins timergesteuert
        mov     CCEN,#10101010b
        orl     A,#00001111b            ;Setzen von P1.0-3 und P4 Portpins
        mov     P1,A                    ;vorbereiten
        mov     P4,#0FFH

        mov     crch,ph_lampe+0
        mov     crcl,ph_lampe+1
        mov     cch1,ph_lampe+2
        mov     ccl1,ph_lampe+3
        mov     cch2,ph_lampe+4
        mov     ccl2,ph_lampe+5
        mov     cch3,ph_lampe+6
        mov     ccl3,ph_lampe+7

        mov     cmh0,ph_lampe+8         ;Compare-Register laden
        mov     cml0,ph_lampe+9
        mov     cmh1,ph_lampe+10
        mov     cml1,ph_lampe+11
        mov     cmh2,ph_lampe+12
        mov     cml2,ph_lampe+13
        mov     cmh3,ph_lampe+14
        mov     cml3,ph_lampe+15

        setb    flag10ms                ;Zeitflag 10ms fr Hauptprogramm
        setb    T2I0                    ;Timer 2 wieder starten
irende  jnb     P3.3,irende             ;Warten, bis High-Pegel

        clr     IE1                     ;IR-Flag loeschen (?!?)
        pop     ACC
        pop     PSW
        reti                            ;fertig.

;****** Hauptprogramm: **************************************************
;*                                                                      *
;*      Das Hauptprogramm hat einzig und allein die                     *
;*      ehrenvolle Aufgabe, aus den seriell empfangenen                 *
;*      8-Bit-Datenbytes 16-Bit-Phasenanschnittszeiten                  *
;*      zu gewinnen. Dazu wird eine Datentabelle aus der                *
;*      Datei Kurve.Asm (wird eingelinkt) verwendet.                    *
;*                                                                      *
;************************************************************************

INIT
        mov     SP,#128         ;Stackpointer irgendwo in der Mitte.

;****** Anzeige initialisieren, Baud und Test abfragen ******************

        mov     A,#led3mA
        call    ledanzinit

        call    eeprom_get
        call    init_control
        call    baudcontrol     ;Testen, ob Baudrate eingestellt werden soll
        call    testmodecontrol ;Testen, ob Testmode gewnscht ist

;****** Serielle Schmnittestelle: ***************************************

        call    setbaudrate

        mov     TMOD,#00100000b         ;Timer 1: Auto-Reload Timer-Mode
        setb    TR1                     ;Starten von Timer 1.
        mov     PCON,#0                 ;SMOD nullsetzen
        setb    ES                      ;Seriellen IR freigeben
        mov     IP1,#00010000b          ;SIO auf Priority-Level 2

;****** Softstart-Variablen: ********************************************

        mov     R0,#lampe
        mov     R7,#8
ilnxt
        mov     @R0,#0
        inc     R0
        djnz    R7,ilnxt

        mov     overheat,#255
        mov     softstart,#0
        call    rechne

;****** Timer und Interrupts: *******************************************

        mov     T2CON,#00000101b        ;Init Timer 2
        mov     CMEN,#0FFH              ;Timersystem auf 12 Compare-Outputs
        mov     CMSEL,#0H               ;einrichten
        mov     CCEN,#10101010b

        clr     IT1                     ;INT-1 pegelgesteuert
        setb    EX1                     ;INT-1 freigeben
        setb    EA

;****** Hauptprogramm-Endlosschleife: ***********************************

main
        dowatch
        jnb     flag10ms,no10ms         ;Tastatur und Display nur
        clr     flag10ms
        call    tastatur                ;alle 10ms bearbeiten.
        call    display

        mov     A,l_blink
        add     A,#80
        mov     l_blink,A
        jnc     no_i_blink
        inc     blink                   ;Blinkzaehler
no_i_blink

        jbc     frame_ok,f_ok
        ljmp    no_f_ok
f_ok    mov     frame_count,#20
no_f_ok
        jbc     data_received,d_rec
        ljmp    no_d_rec
d_rec   mov     data_count,#20
no_d_rec
        djnz    frame_count,no_f_count
        mov     frame_count,#1
no_f_count
        djnz    data_count,no_d_count
        mov     data_count,#1
no_d_count

        mov     A,l_softstart           ;5 Sekunden Softstart forcieren:
        add     A,#80
        mov     l_softstart,A
        jnc     nosoftinc
        mov     A,softstart
        add     A,#1
        jc      nosoftinc
        mov     softstart,A
nosoftinc
no10ms
        call    rechne                  ;aktuelle Phasenwerte rechnen
        ljmp    main

;****** Unterprogramm Rechne: *******************************************

rechne
        mov     R7,#8                   ;R7 ist Schleifenzaehler
        mov     p_lampe,#lampe
        mov     p_phase,#ph_lampe
        mov     p_preheat,#preheat
        mov     p_kurve,#kurve
getnext

        extern  prozent

        mov     R1,p_preheat            ;Testen auf Preheat:
        mov     A,@R1
        mov     dptr,#prozent           ;Wert aus Prozent-Tabelle holen
        movc    A,@A+dptr
        mov     R1,A                    ;und in R1 zwischenspeichern

        mov     R0,p_lampe              ;Lampen-Empfangswert in den Akku
        mov     A,@R0
        clr     C
        subb    A,R1                    ;Preheat davon abziehen
        mov     A,@R0
        jnc     nopreheat               ;wenn Preheat grsser war, wird
        mov     A,R1                    ;Preheat zurckgeliefert.
nopreheat

        mov     R0,param
        cjne    R0,#ptest,r_notest   ;bei Testmode: Rampe auf Ausgnge
        mov     A,R7
        rl      A
        rl      A
        rl      A
        rl      A
        rl      A
        add     A,blink
r_notest
        push    Acc                     ;Testen auf Softstart:
        clr     C
        subb    A,softstart
        pop     Acc
        jc      nosoftstart
        mov     A,softstart
nosoftstart

        mov     R0,A                    ;Lampenwert nach R0
        mov     R1,p_kurve              ;Kurve nach A
        mov     A,@R1
        call    getphase

        mov     R1,p_phase
        mov     A,R3
        clr     EX1                     ;16-Bit-Wert abspeichern:
        mov     @R1,A                   ;dazu 10ms-IR kurzfristig sperren
        mov     A,R2
        inc     R1
        mov     @R1,A
        setb    EX1                     ;10ms-IR wieder freigeben

        inc     p_lampe                 ;alle Pointer weiterstellen:
        inc     p_phase
        inc     p_phase
        inc     p_preheat
        inc     p_kurve

        djnz    R7,getnext              ;Achtmal wiederholen.
        ret

;****** Phasenwert holen: ***********************************************

        extern  phasenkurve
getphase
        rl      A
        add     A,#high phasenkurve     ;im Akku steht Kurven-Nummer
        mov     dph,A
        mov     A,R0                    ;in R0 steht Lampenwert
        clr     C
        rlc     A                       ;schieben wegen Wort-Breite
        jnc     xnothigh
        inc     dph
xnothigh
        mov     dpl,A
        mov     A,#0                    ;Bytes aus Eprom-Speicher holen:
        movc    A,@A+dptr
        mov     R2,A                    ;Low-Byte nach R2
        inc     dptr
        mov     A,#0
        movc    A,@A+dptr
        mov     R3,A                    ;High-Byte nach R3
        ret

;****** Sio initialisieren / Baudrate einstellen: ***********************

setbaudrate
        mov     PSW,#SIOBANK    ;Fehlerstatus des SIO-Handlers
        mov     R0,#lampe+8     ;forcieren
        mov     PSW,#NORMBANK   ;(kein Empfang bis naechstes Break)

        mov     A,baudrate
        cjne    A,#baud250,tbaud125
        mov     SCON,#10010000b         ;Schnittstelle auf Mode 2 (11 Bit)
        mov     PCON,#80h               ;SMOD setzen
tbaud125
        cjne    A,#baud125,tbaud62
        mov     SCON,#10010000b         ;Schnittstelle auf Mode 2 (11 Bit)
        mov     PCON,#0                 ;SMOD nullsetzen
tbaud62
        cjne    A,#baud62,tbaud31
        mov     TH1,#0FFH               ;8Mhz / 12 / 16 = 41.666 kBaud
        mov     SCON,#11010000b         ;Schnittstelle auf Mode 3 (11 Bit)
        mov     PCON,#80h               ;SMOD setzen
tbaud31
        cjne    A,#baud31,tbaudxxx
        mov     TH1,#0FFH               ;8Mhz / 12 / 32 = 20.833 kBaud
        mov     SCON,#11010000b         ;Schnittstelle auf Mode 3 (11 Bit)
        mov     PCON,#0                 ;SMOD nullsetzen
tbaudxxx
        ret


;****** Daten Hauptprogramm: ********************************************

                dseg
bits segment    bit

flag10ms        ds      1       ;10ms-Timeflag an HP
frame_ok        ds      1       ;Error: Fehlerhafter Frame oder Baudrate
data_received   ds      1       ;Daten-OK-Meldung an HP

hpdata segment  intern at 30h

accustack       ds      1       ;Schneller "Stack" fr SIO-Handler

ph_lampe        ds      16      ;Phasenanschnittswerte (Word)
lampe           ds      8       ;Empfangene Lampen-Kanal-Bytes (0..255)

softstart       ds      1       ;255 normal, nach Einschalten von 0 auf 255
l_softstart     ds      1       ;Low-Byte von Softstart
overheat        ds      1       ;255 normal, <255 bei berhitzung
blink           ds      1       ;wird alle 10ms inkrementiert
l_blink         ds      1       ;Low-Byte von Blink
frame_count     ds      1       ;Abwrtszhler fr Frame-Error
data_count      ds      1       ;Abwrtszhler fr Data Received

p_lampe         ds      1       ;Pointer-Platzhalter
p_phase         ds      1
p_preheat       ds      1
p_kurve         ds      1

        end