;****** EEPROM-Ein-/Ausgaberoutinen-Sammlung ********************
;*                                                              *
;*      Ulm, den 3. Dezember 1991 (Sk)                          *
;*                                                              *
;*      Zum Betrieb der seriellen EEPROM's                      *
;*      NMC9306, COP494, und andere.                            *
;*      Benutzt werden 3 Leitungen fuer Chip-Select,            *
;*      Clock und Daten-Input/Output.                           *
;*                                                              *
;*      Fuer den Benutzer stehen die Routinen sio_read und      *
;*      sio_write zur Verfuegung, die jeweils ein Wort          *
;*      (2 Byte) lesen bzw. schreiben.                          *
;*      Das Ver- und Entriegeln des EEPROM-Datenschutzes        *
;*      wird dabei automatisch vorgenommen.                     *
;*      Beim Schreiben werden 20ms Pause eingelegt, bis das     *
;*      EEPROM die empfangenen Daten programmiert hat.          *
;*                                                              *
;*      Nach einem Reset sollte die Routine sio_init            *
;*      benutzt werden, um definierte Leitungspegel             *
;*      einzustellen.                                           *
;*                                                              *
;****************************************************************

;****** Externe Datei: 80517.Asm (eingebunden) ******************
        include 80517.asm

;****** Leseroutine vom EEPROM: sio_read ************************
;*                                                              *
;*      Aufruf mit:                                             *
;*      R7:     Adresse des zu lesenden EEPROM-Worts.           *
;*                                                              *
;*      Rueckgabe von:                                          *
;*      R6:     High-Byte des gelesenen EEPROM-Worts.           *
;*      R5:     Low-Byte des gelesenen EEPROM-Worts.            *
;*                                                              *
;****************************************************************

        public  sio_read

;****** Schreibroutine ins EEPROM: sio_write ********************
;*                                                              *
;*      Aufruf mit:                                             *
;*      R7:     Adresse des zu schreibenden EEPROM-Worts.       *
;*      R6:     High-Byte des zu schreibenden EEPROM-Worts.     *
;*      R5:     Low-Byte des zu schreibenden EEPROM-Worts.      *
;*                                                              *
;*      Rueckgabe: keine                                        *
;*                                                              *
;****************************************************************

        public  sio_write

;****** Initialisierungsroutine der EEPROM-Leitungen: ***********
;*                                                              *
;*      Aufruf:    keine Parameter                              *
;*      Rueckgabe: keine                                        *
;*                                                              *
;****************************************************************

        public  sio_init

;****** Ein-/Ausgabepins fuer serielle EEPROM-Verbindung: *******
;*                                                              *
;*      Um die EEPROM-Software an andere Hardware anzupassen,   *
;*      muessen die folgenden 3 Pin-ZUordnungen ev. geaendert   *
;*      werden.                                                 *
;*      Andere Aenderungen in der Software sind nicht           *
;*      erforderlich!                                           *
;*                                                              *
;****************************************************************

EE_CS   equ     P1.5            ;Belegung auf der 80517-CAT-Karte
EE_SK   equ     P1.6
EE_DAT  equ     P1.7            ;DI und DO auf einem Anschluss


;****** Assembler-Macros: ***************************************

dowatch macro
        setb    IEN0.6                  ;Setzen Bit WDT
        setb    IEN1.6                  ;Setzen Bit SWDT
        endm

clk_puls macro
        setb    EE_SK
        nop
        clr     EE_SK
        endm

sio_start macro
        clr     EE_DAT
        clr     EE_SK
        clr     EE_CS
        setb    EE_CS
        clk_puls
        setb    EE_DAT
        clk_puls
        endm

sio_end macro
        clr     EE_CS
        endm

wait20ms macro
        local   loop1,loop2
        push    AR7
        push    AR6
        mov     R7,#60
loop2   mov     R6,#111
loop1   djnz    R6,loop1
        dowatch
        djnz    R7,loop2
        pop     AR6
        pop     AR7
        endm

;****** Unterprogramme: (global verfuegbar) *********************

sio_read
        sio_start
        mov     A,R7                    ;Adresse von R7 in den Akku
        orl     A,#10000000b
        call    sio_send
        setb    EE_DAT
        call    sio_rec
        mov     R6,A                    ;High-Byte nach R6 abspeichern
        call    sio_rec
        mov     R5,A                    ;Low-Byte nach R5 abspeichern
        sio_end
        ret

sio_write
        push    AR2
        push    AR3
        push    AR4

        mov     A,R6
        mov     R4,A
        mov     A,R5
        mov     R3,A
        call    sio_read
        mov     A,R6
        cjne    A,AR4,dowrite
        mov     A,R5
        cjne    A,AR3,dowrite
        ljmp    nowrite
dowrite
        mov     R2,#3                   ;Bei Fehler: max. 3 Schreibversuche
tryagain
        call    sio_ewen
        mov     A,R7
        call    sio_erase
        sio_start
        mov     A,R7                    ;Adresse von R7 in den Akku
        orl     A,#01000000b
        call    sio_send
        mov     A,R4                    ;High-Byte von R4 in den Akku
        call    sio_send
        mov     A,R3                    ;Low-Byte von R3 in den Akku
        call    sio_send
        sio_end
        wait20ms
        call    sio_ewds

        call    sio_read
        mov     A,R6
        cjne    A,AR4,wrerror
        mov     A,R5
        cjne    A,AR3,wrerror
        ljmp    noerror
wrerror
        djnz    R2,tryagain
noerror
nowrite
        mov     A,R4
        mov     R6,A
        mov     A,R3
        mov     R5,A
        pop     AR4
        pop     AR3
        pop     AR2
        ret

sio_init
        clr     EE_DAT
        clr     EE_SK
        clr     EE_CS
        ret

;****** Unterprogramme: (lokal in dieser Library) ***************

sio_erase
        sio_start
        orl     A,#11000000b
        call    sio_send
        sio_end
        wait20ms
        ret

sio_ewen
        sio_start
        mov     A,#00110000b
        call    sio_send
        sio_end
        ret

sio_ewds
        sio_start
        mov     A,#0
        call    sio_send
        sio_end
        ret

sio_send
        push    AR7
        mov     R7,#8
nxt_sbit
        rlc     A
        mov     EE_DAT,C
        clk_puls
        djnz    R7,nxt_sbit
        setb    EE_DAT
        pop     AR7
        ret

sio_rec
        push    AR7
        mov     R7,#8
nxt_rbit
        clk_puls
        mov     C,EE_DAT
        rlc     A
        djnz    R7,nxt_rbit
        pop     AR7
        ret

;****** Library-Ende ********************************************

        end