;************** IIC-Bus Ein-/Ausgaberoutinen-Library ************************
;                                                                           *
;               Ulm, den 3. Dezember 1991 (SK)                              *
;                                                                           *
;               Zum Betrieb von IIC-Bus-Bausteinen an einem                 *
;               8051-Controller (oder: 80535,80537).                        *
;                                                                           *
;               Benutzt werden nur 2 Leitungen fuer Clock und Daten.        *
;                                                                           *
;               Fuer den Benutzer stehen die beiden Routinen                *
;               Write_IIC und Read_IIC zur Verfuegung, die jeweils ein      *
;               Byte lesen, bzw. schreiben.                                 *
;                                                                           *
;****************************************************************************

;************** Routine Read_IIC ********************************************
;                                                                           *
;               Liest ein Byte aus dem Uhren-Ram an der Adresse R0          *
;               und gibt es im Akku zurueck.                                *
;                                                                           *
;Aufruf mit:    R0 : Adresse des zu lesenden Bytes im Uhren-Ram.            *
;                                                                           *
;Rueckgabe:     Akku : Wert des gelesenen Bytes.                            *
;                                                                           *
;Benutzte Regs: R2                                                          *
;                                                                           *
;****************************************************************************

        public  read_iic

;************** Routine Write_IIC *******************************************
;                                                                           *
;               Schreibt ein im Akku uebergebenes Byte ins Uhren-Ram        *
;               an die Adresse in R0.                                       *
;                                                                           *
;Aufruf mit:    Akku : Zu schreibendes Byte                                 *
;               R0 : Adresse des zu schreibenden Bytes im Uhren-Ram.        *
;                                                                           *
;Rueckgabe:     Keine Parameter.                                            *
;                                                                           *
;Benutzte Regs: R2, R1                                                      *
;                                                                           *
;****************************************************************************

        public  write_iic

;************** Hardeware-Konstanten: ***************************************
;                                                                           *
;               Bei Hardware-Aenderung oder beim Weiterverwenden der        *
;               Software auf anderen Platinen muss nur hier geaendert       *
;               werden!                                                     *
;                                                                           *
;****************************************************************************

icadr   equ     01110000B    ; Hardware-Adresse LED-Chip
clkbit  equ     p3.5         ; Clock-Anschlu am 8051 - Processor
datbit  equ     p1.4         ; Daten-Anschlu am 8051-Processor


;************** Routine write_iic (Public) **********************************
write_iic
        mov     r1,a            ;Wert in R1 sichern
        acall   start           ;Hier fngt eigendliche Routine Write an.
        mov     a,#icadr
        acall   bsend
        mov     a,r0            ;Adresse im Uhren-RAm setzen
        acall   bsend
        mov     a,r1            ;Wert ins Uhren-Ram schreiben
        acall   bsend
        acall   stop            ;Uebertragung terminieren
        ret

;************** Routine Read_IIC (Public) ***********************************
read_iic
        acall   start           ;Hier fngt eigendliche Routine Read an.
        mov     a,#icadr
        acall   bsend
        mov     a,r0            ;Adresse im Uhren-RAm stellen
        acall   bsend
        acall   stop
        acall   start
        mov     a,#(icadr or 1) ;Aus Uhren-Ram lesen !
        acall   bsend
        acall   bempf           ;Wert wird im Akku zurueckgeliefert.
        acall   stop
        ret

;************** Byte auf IIC-Bus ausgeben: **********************************
bsend                           ;im Akku steht zu sendendes Byte:
        mov     r2,#8           ;R2 ist Schleifenzhler
snext   rlc     a               ;Akku links rum in Carry rotieren
        mov     datbit,c        ;und Carry aufs Datenbit ausgeben.
        nop                     ;kompletten Clockimpuls ausgeben
        nop
        setb    clkbit
        nop
        nop
        nop
        nop
        clr     clkbit
        djnz    r2,snext        ;achtmal das ganze
        setb    datbit          ;Acknowledge:
        nop                     ;kompletten Clockimpuls ausgeben
        nop
        setb    clkbit
        nop
        nop
        nop
        nop
        clr     clkbit
        ret                     ;und fertig.

;************** Byte vom IIC-Bus lesen: *************************************
bempf                           ;empfangenes Byte ist danach im Akku:
        mov     r2,#8           ;R2 ist Schleifenzhler
        mov     a,#0            ;Akku mit 0 vorbelegen
enext   nop
        nop
        nop
        setb    clkbit          ;positiver Clockpuls
        nop
        nop
        mov     c,datbit        ;Datenbit ins Carry
        rlc     a               ;und dem Akku hinten reinschieben.
        clr     clkbit          ;negativer Clockpuls.
        djnz    r2,enext        ;achtmal das Ganze
        nop
        setb    datbit          ;kein Acknowledge:
        nop                     ;kompletten Clockimpuls ausgeben
        nop
        setb    clkbit
        nop
        nop
        nop
        nop
        clr     clkbit
        ret

;************** Start-Zeichen auf IIC-Bus ausgeben:**************************
start   setb    datbit
        setb    clkbit
        nop
        nop
        nop
        nop
        clr     datbit
        nop
        nop
        nop
        nop
        nop
        clr     clkbit
        ret

;************** Stop-Zeichen auf IIC-Bus ausgeben:***************************
stop    clr     datbit
        setb    clkbit
        nop
        nop
        nop
        nop
        setb    datbit
        ret

;************** Library-Ende ************************************************
        end