;****** Anzeige- und Tastaturmodul Lichtendstufe ************************
;*                                                                      *
;*      Ulm, den 3. Dezember 1991 (Sk)                                  *
;*                                                                      *
;*      Release: Mnchen, den 30.03.92 (SK)                             *
;*                                                                      *
;*                                                                      *
;*      Bearbeitet die Tastatur und das Display der                     *
;*      Lichtendstufen.                                                 *
;*      Benutzt werden Unterprogramme aus den Library's                 *
;*      IIC.ASM und EEPROM.ASM, diese muessen dazugelinkt               *
;*      werden.                                                         *
;*                                                                      *
;************************************************************************

;****** Externe Datei: 80517.Asm (eingebunden) **************************

        include x80517.asm
        include macros.asm
        include konstanten.asm

;****** Externe Unterprogramme: *****************************************

        extern  write_iic,read_iic
        extern  sio_write,sio_read
        extern  setbaudrate

;****** ffentliche Unterprogramme/Variablen: ***************************

        public  eeprom_get              ;Einstellungen aus EEPROM holen
        public  init_control            ;Einstellungen kontrollieren
        public  baudcontrol             ;Editmode Baudrate
        public  testmodecontrol         ;Editmode Testen forcieren
        public  ledanzinit              ;Strom fr LedAnzeige einstellen
        public  tastatur                ;Tastaturbearbeitung
        public  display                 ;Display-Darstellung

        public  kanal                   ;Externe Variablen (aus EEPROM)
        public  preheat
        public  kurve
        public  baudrate
        public  param

;****** Variablen-Initkontrolle: ****************************************
;*                                                                      *
;*      * Setzt Arbeitsvariablen auf Null                               *
;*      * Kontrolliert Baudrate                                         *
;*      * Kontrolliert Kanalnummer                                      *
;*      * Kontrolliert Preheat-Variablentabelle                         *
;*      * Kontrolliert Kurven-Variablentabelle                          *
;*                                                                      *
;*      * Bei fehlerhaften Variablen wird die entsprechende             *
;*        Variable auf Null gesetzt.                                    *
;*                                                                      *
;************************************************************************

init_control
        mov     param,#0        ;Arbeitsvariablen nullsetzen
        mov     t_aktiv,#0
        mov     lampnr,#0
        mov     altkey,#0

        mov     a,#maxbaudrate  ;Eingestellte Baudrate kontrollieren
        clr     C
        subb    A,baudrate
        jnc     ov_baudrate
        mov     baudrate,#0
ov_baudrate
        mov     A,#maxkanal     ;Eingestellte Kanalnummer kontrollieren
        clr     C
        subb    A,kanal
        jnc     ov_kanal
        mov     kanal,#0
ov_kanal
ov_lampe
        mov     R7,#8           ;R7 ist Lampenzaehler
        mov     R0,#preheat     ;Zeiger auf Preheat-Variablen
        mov     R1,#kurve       ;Zeiger auf Kurven-Variablen
nextcontrol
        mov     A,#maxpreheat   ;Maximalwert mit A vergleichen
        clr     C
        subb    A,@R0
        jnc     ov_preheat
        mov     A,#0
        mov     @R0,A
ov_preheat
        mov     A,#maxkurve     ;Maximalwert mit A vergleichen
        clr     C
        subb    A,@R1
        jnc     ov_kurve
        mov     A,#0
        mov     @R1,A
ov_kurve                        ;Naechsten Variablen-Tabelleneintrag
        inc     R0
        inc     R1
        djnz    R7,nextcontrol  ;Noch eine Variable?
        ret                     ;Nein, Kontrolle beendet.

;****** Unterprogramme: *************************************************
;*                                                                      *
;*                                                                      *
;************************************************************************

baudcontrol
        call    getcode                 ;Tastaturcode holen
        cjne    A,#kbaud,e_baud         ;testen
        mov     t_aktiv,#t_sleep        ;Tastatur aktivieren
        mov     param,#pbaud
        mov     tcount,#tslow
        mov     xcount,#sleeptime
        mov     altkey,#0

        mov     A,#led21mA
        call    ledanzinit

        mov     ledanz+2,#l_baud
        mov     ledanz+0,#strich
        mov     ledanz+1,#strich
        call    wr_disp
        call    w10ms
baud_released
        call    getcode
        jnz     baud_released
        call    pause1s
e_baud  ret

testmodecontrol
        call    getcode                 ;Tastaturcode holen
        cjne    A,#ktest,e_test         ;testen
        mov     t_aktiv,#0              ;Tastatur schlaeft!!!
        mov     param,#ptest
        mov     tcount,#tslow
        mov     xcount,#sleeptime
        mov     altkey,#0

        mov     A,#led21mA
        call    ledanzinit

        mov     ledanz+2,#l_test
        mov     ledanz+0,#strich
        mov     ledanz+1,#strich
        call    wr_disp
        call    w10ms
test_released
        call    getcode
        jnz     test_released
        call    pause1s
e_test  ret

;****** Pausenroutine (zum Testen) **************************************

pause1s
        mov     r7,#100
nxtp4s  push    ar7
        call    w10ms
        pop     ar7
        djnz    r7,nxtp4s
        ret

w10ms   mov     R5,#15          ;10ms Pausenroutine
l3      mov     R6,#1
l2      mov     R7,#222
l1      djnz    R7,l1
        dowatch
        djnz    R6,l2
        djnz    R5,l3
        ret

;****** Pointer und Maximalwert fuer Variable holen: ********************

getvpointer
        push    AR2
        mov     R2,param
        cjne    R2,#plampnr,vnolampnr
        mov     A,#lampnr
        mov     R7,#maxlampnr
        ljmp    ptogh
vnolampnr
        cjne    R2,#ppreheat,vnopreheat
        mov     A,#preheat
        add     A,lampnr
        mov     R7,#maxpreheat
        ljmp    ptogh
vnopreheat
        cjne    R2,#pkurve,vnokurve
        mov     A,#kurve
        add     A,lampnr
        mov     R7,#maxkurve
        ljmp    ptogh
vnokurve
        cjne    R2,#pbaud,vnobaud
        mov     A,#baudrate
        mov     R7,#maxbaudrate
        ljmp    ptogh
vnobaud
        mov     A,#kanal                ;den Akku,
        mov     R7,#maxkanal            ;max. Groesse nach R7.
ptogh
        mov     R0,A
        pop     AR2
        ret

;****** EEPROM-Schreib-/Leseroutinen: ***********************************
;*                                                                      *
;*                                                                      *
;************************************************************************

;****** Lesen aus dem EEPROM / Sichern ins EEPROM: **********************

getepr  macro   d_adr,e_adr
        mov     R7,#e_adr
        call    sio_read
        mov     d_adr,R6
        mov     d_adr+1,R5
        endm

saveepr macro   d_adr,e_adr
        mov     R7,#e_adr
        mov     R6,d_adr
        mov     R5,d_adr+1
        call    sio_write
        endm

eeprom_get
        getepr  kanal,ekanal
        getepr  preheat,epreheat
        getepr  preheat+2,epreheat+1
        getepr  preheat+4,epreheat+2
        getepr  preheat+6,epreheat+3
        getepr  kurve,ekurve
        getepr  kurve+2,ekurve+1
        getepr  kurve+4,ekurve+2
        getepr  kurve+6,ekurve+3
        ret

eeprom_save
        saveepr kanal,ekanal
        saveepr preheat,epreheat
        saveepr preheat+2,epreheat+1
        saveepr preheat+4,epreheat+2
        saveepr preheat+6,epreheat+3
        saveepr kurve,ekurve
        saveepr kurve+2,ekurve+1
        saveepr kurve+4,ekurve+2
        saveepr kurve+6,ekurve+3
        ret

;****** Tastaturbearbeitung: ********************************************
;*                                                                      *
;*                                                                      *
;************************************************************************

tastatur
        call    getkey
        jz      nokey
        mov     R2,param
        cjne    R2,#ptest,tnotest
        mov     param,#0
        ljmp    led_3mA
tnotest
        mov     R2,t_aktiv
        cjne    R2,#0,isaktiv           ;Tastatur schon aktiviert ?
        cjne    A,#kscroll,nokey        ;nur die Scrolltaste kann wecken
        mov     t_aktiv,#t_sleep        ;Tastatur aktivieren:

        mov     A,#led21mA
        call    ledanzinit

        mov     param,#1                ;Parameter = Kanal einstellen und
        ljmp    nokey                   ;ersten Tastendruck vergessen.
isaktiv
        mov     R2,A                    ;Tastenwert zwischenspeichern in R2
        mov     t_aktiv,#t_sleep
        cjne    R2,#kscroll,nokscroll   ;Taste "Scrollen" ?
        mov     A,param
        inc     A                       ;Scrollen zwischen 1 und 5

        push    ACC
        clr     C
        subb    A,#5
        pop     ACC
        jc      pnotup
        mov     A,#1
pnotup
        mov     param,A
        ljmp    nokey
nokscroll                               ;Variable bearbeiten:
        call    getvpointer             ;Pointer auf Variable nach R0
        cjne    R2,#kup,nokup           ;Taste "Up" ?
        mov     A,@R0
        mov     puffer,R7
        cjne    A,puffer,doinc
        mov     A,#0ffH                 ;obere Grenze: Null zurueckspeichern
doinc   inc     A                       ;Grenze nicht erreicht: incrementieren
        mov     @R0,A
        ljmp    toghupdown
nokup
        cjne    R2,#kdown,nokdown    ;Taste "Down" ?
        mov     A,@R0
        cjne    A,#0,dodec
        mov     A,R7
        inc     A
dodec   dec     A
        mov     @R0,A
        ljmp    toghupdown
nokdown
        mov     A,#0                    ;Taste "Zero" !!!
        mov     @R0,A
toghupdown
        mov     A,param
        cjne    A,#pbaud,nokey
        call    setbaudrate
nokey
        mov     R2,t_aktiv
        cjne    R2,#0,keyready          ;ins EEPROM wird nur gespeichert,
        mov     A,param                 ;wenn der Aktivzaehler Null und
        jz      keyready                ;der Parameter-Zaehler ungleich Null
        cjne    A,#ptest,t_eesave
        ljmp    keyready
t_eesave
        mov     param,#0                ;ist !!!
        mov     R0,#1
        mov     A,#strich               ;Waehrend dem Sichern ins EEPROM
        call    write_iic               ;erscheinen auf dem Display
        mov     R0,#2                   ;zwei Bindestriche.
        mov     A,#strich
        call    write_iic
        call    eeprom_save
led_3mA
        mov     A,#led3mA
        call    ledanzinit

keyready
        ret

;****** Taste anfragen und Tastatur-Wakeup-Counter stellen **************

getkey
        djnz    xcount,isnull
        mov     A,t_aktiv
        jz      isnull
        dec     t_aktiv
isnull
        call    keyboard
        jz      getnokey
        mov     xcount,#sleeptime
getnokey
        ret

;****** Tastaturbearbeitung: ********************************************
;*                                                                      *
;*      Liefert einen Tastendruck von der Tastatur zurueck.             *
;*      Dabei wird das Tastenprellen eliminiert und eine Umschaltung    *
;*      zwischen langsamem Tasten und Schnelldurchlauf realisiert.      *
;*                                                                      *
;************************************************************************

getcode
        dowatch
        movx    A,@dptr         ;Tastendaten vom Bus holen
        cpl     A               ;Matrixnegation aufheben
        anl     A,#kmask        ;3 Tasten ausmaskieren
        ret

keyboard                        ;Taste holen (Macro)
        call    getcode
        jz      dokexit                 ;Wenn keine Taste gedrueckt -> Ende
        cjne    A,altkey,diffkey        ;Neue Taste oder Wiederholung?

equkey  djnz    tcount,dokex2           ;Wiederholung: Teste Zeitzaehler
        mov     tcount,#tspeed          ;Naechste Taste mit kurzer Zeitkonst.
        ljmp    dokexit
diffkey
        cjne    A,#kzero,diff2
        ljmp    dokneu
diff2   mov     R7,altkey
        cjne    R7,#0,dokex2
dokneu
        mov     tcount,#tslow           ;Taste unterschiedlich, lange Zeitk.
dokexit mov     altkey,A                ;Neue Taste merken fuer naechstesmal.
        ret
dokex2
        mov     A,#0
        ret

;****** Displayausgabe der aktuellen Parameter: *************************
;*                                                                      *
;*                                                                      *
;*                                                                      *
;*                                                                      *
;*                                                                      *
;************************************************************************

ledanzinit                      ;im Akku steht einzustellender Strom
        mov     R0,#0           ;Initregister adressieren
        call    write_iic       ;Initialisierung an LED-Treiber senden

display
        call    getvpointer
        mov     A,@R0
        mov     R2,param                ;Testen ob Parameter Lampen-Nr.
        cjne    R2,#plampnr,nolampe     ;dargestellt wird.
        inc     A                       ;Lampen-Nr. faengt bei 1 an.
        mov     R7,#l_lampnr
        ljmp    dtogh
nolampe
        cjne    R2,#ppreheat,nopreheat  ;Testen,ob Parameter Preheat
        mov     R7,#l_preheat           ;dargestellt wird.
        ljmp    dtogh
nopreheat
        cjne    R2,#pkurve,nokurve      ;Testen, ob Parameter Kurve
        inc     A                       ;dargestellt wird.
        mov     R7,#l_kurve
        ljmp    dtogh
nokurve
        cjne    R2,#pkanal,noaktkan     ;Testen, ob Parameter Kanal
        mov     R7,#l_kanal             ;dargestellt wird.
        cjne    A,#0,dtogh              ;(aktiv)
        mov     A,#100
        ljmp    dtogh
noaktkan
        cjne    R2,#0,nopaskan          ;Testen, ob Parameter Kanal
        mov     R7,#l_passiv            ;dargestellt wird.
        cjne    A,#0,dtogh              ;(passiv)
        mov     A,#100
        ljmp    dtogh
nopaskan
        cjne    R2,#pbaud,nobaud        ;Testen, ob Parameter Baudrate
        mov     R7,#l_baud              ;dargestellt wird.
        add     A,#101
        ljmp    dtogh
nobaud
        cjne    R2,#ptest,notest        ;Testen, ob Testmode aktiv
        mov     R7,#l_test              ;ist.
        mov     A,#105
        ljmp    dtogh
notest
dtogh                                   ;Ab hier fuer alle Parameter:
        mov     ledanz+2,R7             ;in R7 steht Code fuer Auswahl-LED
        call    getdisp                 ;holt fuer Zahl in A das entsprechende
        mov     ledanz,R7               ;LED-Bitmuster nach R6 und R7.
        mov     ledanz+1,R6

powerled   equ  Acc.0
dataled    equ  Acc.1
frameled   equ  Acc.2
;errorled   equ  Acc.3

        extern  frame_count:byte
        extern  data_count:byte
        extern  blink:byte
wr_disp
        mov     A,ledanz
        mov     R0,#2                  ;Darstellen der LED-Digits 1 und 2:
        call    write_iic
        mov     R0,#1
        mov     A,ledanz+1
        call    write_iic

        mov     A,ledanz+2              ;Led-Digit 3 um Bits erweitern:
        mov     R0,#3
        call    write_iic

        mov     A,blink                 ;Blink-Bit ins Carry
        jb      Acc.6,set_c
        jb      Acc.5,set_c
        jb      Acc.4,set_c
        jb      Acc.3,set_c
        jb      Acc.2,set_c
        clr     C
        ljmp    notset_c
set_c
        setb    C
notset_c
        mov     A,#0
        mov     powerled,C              ;Bits des Status-Led-Bytes setzen
        mov     R0,frame_count
        setb    frameled
        cjne    R0,#1,nfe
        clr     frameled
nfe
        mov     R0,data_count
        setb    dataled
        cjne    R0,#1,ndr
        clr     dataled
ndr
        mov     R0,#4
        call    write_iic
        ret


;****** Displaydarstellung: *********************************************
;*                                                                      *
;*                                                                      *
;*                                                                      *
;************************************************************************

getdisp                         ;Holt das der Zahl im Akku entsprechende
        mov     dptr,#disptab   ;Display-Bitmuster in die Register
        rl      A               ;R7 (High) und R6 (Low).
        push    ACC
        movc    A,@A+dptr
        mov     R7,A
        pop     ACC
        inc     A
        movc    A,@A+dptr
        mov     R6,A
        ret

strich  equ     00000100b       ;Mittelstrich

;__      equ     0
;_1      equ     01001000b
;_2      equ     00111101b
;_3      equ     01101101b
;_4      equ     01001011b
;_5      equ     01100111b
;_6      equ     01110111b
;_7      equ     01001100b
;_8      equ     01111111b
;_9      equ     01101111b
;_0      equ     01111110b
;_A      equ     01011111b
;_E      equ     00110111b
;_t      equ     00110011b

__      equ     0
_1      equ     01000001b
_2      equ     10101101b
_3      equ     01101101b
_4      equ     01000111b
_5      equ     01101110b
_6      equ     11101110b
_7      equ     01001001b
_8      equ     11101111b
_9      equ     01101111b
_0      equ     11101011b
_A      equ     11001111b
_E      equ     10101110b
_t      equ     10100110b

_p2     equ     00111101b       ;Zahlen mit Punkt (nicht bentzt)
_p1     equ     01001000b
_p0     equ     01111110b

l_kanal    equ  00000001b       ;Leuchtdioden-Codes
l_lampnr   equ  00000010b
l_preheat  equ  00000100b
l_kurve    equ  00001000b
l_passiv   equ  00000000b

l_baud     equ  00000011b       ;Led Testmode
l_test     equ  00001100b       ;Led Baudraten-Einstellung

disptab db      __,_0,  __,_1,  __,_2,  __,_3,  __,_4
        db      __,_5,  __,_6,  __,_7,  __,_8,  __,_9
        db      _1,_0,  _1,_1,  _1,_2,  _1,_3,  _1,_4
        db      _1,_5,  _1,_6,  _1,_7,  _1,_8,  _1,_9
        db      _2,_0,  _2,_1,  _2,_2,  _2,_3,  _2,_4
        db      _2,_5,  _2,_6,  _2,_7,  _2,_8,  _2,_9
        db      _3,_0,  _3,_1,  _3,_2,  _3,_3,  _3,_4
        db      _3,_5,  _3,_6,  _3,_7,  _3,_8,  _3,_9
        db      _4,_0,  _4,_1,  _4,_2,  _4,_3,  _4,_4
        db      _4,_5,  _4,_6,  _4,_7,  _4,_8,  _4,_9
        db      _5,_0,  _5,_1,  _5,_2,  _5,_3,  _5,_4
        db      _5,_5,  _5,_6,  _5,_7,  _5,_8,  _5,_9
        db      _6,_0,  _6,_1,  _6,_2,  _6,_3,  _6,_4
        db      _6,_5,  _6,_6,  _6,_7,  _6,_8,  _6,_9
        db      _7,_0,  _7,_1,  _7,_2,  _7,_3,  _7,_4
        db      _7,_5,  _7,_6,  _7,_7,  _7,_8,  _7,_9
        db      _8,_0,  _8,_1,  _8,_2,  _8,_3,  _8,_4
        db      _8,_5,  _8,_6,  _8,_7,  _8,_8,  _8,_9
        db      _9,_0,  _9,_1,  _9,_2,  _9,_3,  _9,_4
        db      _9,_5,  _9,_6,  _9,_7,  _9,_8,  _9,_9

        db      __,_A           ;Analog-Eingang

        db      _2,_5           ;Baudrate = 0,250 Mbaud  (250,00 kb = 25)
        db      _1,_2           ;   "     = 0,125 Mbaud  (125,00 kb = 12)
        db      __,_6           ;   "     = 0,062 Mbaud  ( 62,50 kb =  6)
        db      __,_3           ;   "     = 0,031 Mbaud  ( 31,25 kb =  3)

        db      _t,_t           ;Test-Mode

        db      _E,_E           ;Error-Anzeige

;****** Daten: **********************************************************
;*                                                                      *
;*                                                                      *
;************************************************************************

        dseg
leddata   segment intern at 60h
;  status  ds      1               ;System-Statusanzeige
t_aktiv ds      1               ;Abschaltcounter der Tastatur
param   ds      1               ;aktueller Display-Parameter (0: Display off)
ledanz  ds      3               ;Ledanzeige: Zahl high, Zahl low, Status

lampnr  ds      1               ;Displayparameter Lampennummer

kanal   ds      1               ;DMX-Empfangskanalnummer
baudrate ds     1
preheat ds      8               ;Preheat-Werte der 8 Kanaele
kurve   ds      8               ;entpackte Kurvenbytes (1 Byte pro Kanal)

puffer  ds      1               ;Zwischenspeicher
xcount  ds      1

altkey  ds      1
tcount  ds      1

;****** Daten EEPROM: ***************************************************

ekanal   equ    0               ;Word-Adressen, jeweils 2 Bytes im EEPROM
epreheat equ    1
ekurve   equ    5

        end